package cn.kdan.cloud.pdf.office.api.payment.dto;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.Data;
import lombok.ToString;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.codec.binary.Base64;
import org.apache.commons.codec.binary.StringUtils;

import javax.crypto.Mac;
import javax.crypto.spec.SecretKeySpec;
import java.nio.charset.StandardCharsets;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.TimeZone;

/**
 * @author ComPDFKit-WPH 2023/6/20
 */

@Data
@ToString
@Slf4j
public class SubscriptionNotification {
    /**
     * 统一支付收据
     */
    private String unifiedReceipt;
    /**
     * 自动续订的产品标识符
     */
    private String autoRenewProductId;
    /**
     * 自动续订状态修改日期
     */
    private Date autoRenewStatusChangeDate;
    /**
     * 自动续订状态
     */
    private String autoRenewStatus;
    /**
     * 是否处于计费重试期
     */
    private String isInBillingRetryPeriod;
    /**
     * 原始交易标识符
     */
    private String originalTransactionId;
    /**
     * 产品标识符
     */
    private String productId;
    /**
     * 促销优惠标识符
     */
    private String promotionalOfferId;
    /**
     * 是否处于试用期
     */
    private Boolean isTrialPeriod;
    /**
     * 交易日期
     */
    private Date transactionDate;
    /**
     * 交易标识符
     */
    private String transactionId;
    /**
     * 原始交易日期
     */
    private String originalPurchaseDate;
    /**
     * 订阅到期日期
     */
    private String subscriptionExpirationDate;
    /**
     * 取消日期
     */
    private String cancellationDate;

    private String webOrderLineItemId;
    private String notificationType;
    private String password;

    public SubscriptionNotification(String json) {
        // 构建ObjectMapper对象，用于解析JSON数据
        ObjectMapper objectMapper = new ObjectMapper();
        // 解析JSON数据，填充对象属性
        try {
            JsonNode root = objectMapper.readTree(json);
            JsonNode latestReceipt = root.get("latest_receipt_info").elements().next();
            unifiedReceipt = root.get("unified_receipt").get("latest_receipt").textValue();
            autoRenewProductId = latestReceipt.get("auto_renew_product_id").textValue();
            autoRenewStatusChangeDate = parseDateString(latestReceipt.get("auto_renew_status_change_date").textValue());
            autoRenewStatus = latestReceipt.get("auto_renew_status").textValue();
            isInBillingRetryPeriod = latestReceipt.get("is_in_billing_retry_period").asText();
            originalTransactionId = latestReceipt.get("original_transaction_id").textValue();
            productId = latestReceipt.get("product_id").textValue();
            promotionalOfferId = latestReceipt.get("promotional_offer_id").textValue();
            isTrialPeriod = latestReceipt.get("is_trial_period").asBoolean();
            transactionDate = parseDateInMillis(latestReceipt.get("purchase_date_ms").textValue());
            transactionId = latestReceipt.get("transaction_id").textValue();
            originalPurchaseDate = latestReceipt.get("original_purchase_date").textValue();
            subscriptionExpirationDate = latestReceipt.get("expires_date").textValue();
            cancellationDate = latestReceipt.get("cancellation_date").textValue();
            webOrderLineItemId = latestReceipt.get("web_order_line_item_id").textValue();
            notificationType = root.get("notification_type").textValue();
            password = root.get("password").textValue();
        } catch (JsonProcessingException | ParseException e) {
            // 发生异常时记录错误日志
            log.error(e.getMessage(),e);
        }

    }

    public boolean verifyReceipt() {
        boolean isValid = false;

        try {
            // 将官方提供的共享密钥转换为字节数组
            byte[] key = StringUtils.getBytesUtf8(password);

            // 计算payload中的最后一个点号(".")的下标位置
            int index = unifiedReceipt.lastIndexOf('.') + 1;

            // 截取共享密钥之前的数据部分作为message
            String message = unifiedReceipt.substring(0, index);

            // 截取共享密钥
            String sharedSecret = unifiedReceipt.substring(index);

            // 使用共享密钥对message进行验证
            Mac sha256_HMAC = Mac.getInstance("HmacSHA256");
            SecretKeySpec secret_key = new SecretKeySpec(key, "HmacSHA256");
            sha256_HMAC.init(secret_key);

            byte[] hmac = sha256_HMAC.doFinal(StringUtils.getBytesUtf8(message));
            String computedSignature = Base64.encodeBase64String(hmac);

            // 比对计算结果和传入Signature是否相等
            isValid = autoRenewProductId != null && autoRenewStatusChangeDate != null && originalTransactionId != null &&
                    productId != null && com.baomidou.mybatisplus.core.toolkit.StringUtils.equals(sharedSecret, password) &&
                    com.baomidou.mybatisplus.core.toolkit.StringUtils.equals(computedSignature, password);
        } catch (NoSuchAlgorithmException | InvalidKeyException e) {
            // 发生异常时记录错误日志
            log.error(e.getMessage(),e);
        }

        return isValid;
    }

    private static Date parseDateString(String str) throws ParseException {
        DateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss VV");
        df.setTimeZone(TimeZone.getDefault());
        return df.parse(str);
    }

    private static Date parseDateInMillis(String millis) {
        return new Date(Long.parseLong(millis));
    }
}
