package cn.kdan.cloud.pdf.office.common.config;

import cn.kdan.cloud.pdf.office.common.constant.RabbitMqConstant;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.core.*;
import org.springframework.amqp.rabbit.annotation.EnableRabbit;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import javax.annotation.PostConstruct;

/**
 * @author ComPDFKit-WPH 2023/1/11
 * <p>
 * RabbitMqConfig
 */
@Configuration
@RequiredArgsConstructor
@Slf4j
@EnableRabbit
public class RabbitMqConfig {

    private final RabbitTemplate rabbitTemplate;

    /**
     * 发送回调
     */
    @PostConstruct
    public void publisherConfirm() {
        // 消息抵达服务器确认回调，correlationData：当前消息唯一关联数据（消息的唯一id）,ack：消息是否抵达服务器，cause：失败的原因
        rabbitTemplate.setConfirmCallback(
                (correlationData, ack, cause) -> {
                    log.info("发送成功，correlationData -> {}, ack -> {}, cause -> {}", correlationData, ack, cause);
                }
        );

        // 消息到队列的确认回调（当消息未抵达到队列时才会回调），message：当前发送的消息，replyCode：回复的状态码，replyText：回复的信息，exchange：交换机，routingKey：路由键
        rabbitTemplate.setReturnCallback(
                (message, replyCode, replyText, exchange, routingKey) -> {
                    log.info("发送失败，message -> {}, replyCode -> {}, replyText -> {}, exchange -> {}, routingKey -> {}", message, replyCode, replyText, exchange, routingKey);
                }
        );
    }

    @Bean
    public TopicExchange userLogOffDelayExchange(){
        TopicExchange exchange = new TopicExchange(RabbitMqConstant.USER_LOGOFF_DELAY_EXCHANGE, true, false);
        exchange.setDelayed(true);
        return exchange;
    }
    @Bean
    public Queue userLogOffDelayQueue(){
        return new Queue(RabbitMqConstant.USER_LOGOFF_DELAY_QUEUE, true, false, false);
    }

    @Bean
    public Binding userLogOffDelayBinding() {
        return BindingBuilder.bind(userLogOffDelayQueue()).to(userLogOffDelayExchange()).with(RabbitMqConstant.USER_LOGOFF_ROUTING_KEY);
    }

    // 交换机配置
    /* email交换机 */
    @Bean
    public Exchange emailExchange() {
        return new TopicExchange(RabbitMqConstant.EMAIL_EXCHANGE, true, false);
    }

//    @Bean
//    public TopicExchange paymentExchange() {
//        return new TopicExchange(RabbitMqConstant.PAYMENT_EXCHANGE, true, false);
//    }

    //声明死信交换机
    @Bean
    public TopicExchange paymentDelayExchange(){
        TopicExchange exchange = new TopicExchange(RabbitMqConstant.PAYMENT_DELAY_EXCHANGE, true, false);
        exchange.setDelayed(true);
        return exchange;
    }
    @Bean
    public TopicExchange orderGiftDelayExchange(){
        TopicExchange exchange = new TopicExchange(RabbitMqConstant.ORDER_GIFT_DELAY_EXCHANGE, true, false);
        exchange.setDelayed(true);
        return exchange;
    }

    // 订单关闭
    @Bean
    public Queue paymentOrderDelayQueue(){
        return new Queue(RabbitMqConstant.PAYMENT_ORDER_DELAY_QUEUE, true, false, false);
    }
    @Bean
    public Binding paymentOrderDelayBinding() {
        return BindingBuilder.bind(paymentOrderDelayQueue()).to(paymentDelayExchange()).with(RabbitMqConstant.PAYMENT_ORDER_ROUTING_KEY);
    }

    @Bean
    public Queue orderGiftDelayQueue(){
        return new Queue(RabbitMqConstant.ORDER_GIFT_DELAY_QUEUE, true, false, false);
    }
    @Bean
    public Binding OrderGiftDelayBinding() {
        return BindingBuilder.bind(orderGiftDelayQueue()).to(orderGiftDelayExchange()).with(RabbitMqConstant.ORDER_GIFT_DELAY_ROUTING_KEY);
    }

    // AppStore验证票据重试处理相关
    @Bean
    public Queue appStoreVerifyRetryDelayQueue(){
        return new Queue(RabbitMqConstant.APPSTORE_VERIFY_RECEIPT_RETRY_QUEUE, true, false, false);
    }
    @Bean
    public Binding appStoreVerifyRetryDelayBinding() {
        return BindingBuilder.bind(appStoreVerifyRetryDelayQueue()).to(paymentDelayExchange()).with(RabbitMqConstant.APPSTORE_VERIFY_RECEIPT_RETRY_ROUTING_KEY);
    }
//    @Bean
//    public Queue paymentOrderQueue() {
//        Map<String, Object> params = new HashMap<>();
//        // x-dead-letter-exchange 声明了队列里的死信转发到的DLX名称，
//        params.put("x-dead-letter-exchange", RabbitMqConstant.DELAY_EXCHANGE);
//        // x-dead-letter-routing-key 声明了这些死信在转发时携带的 routing-key 名称。
//        params.put("x-dead-letter-routing-key", RabbitMqConstant.PAYMENT_ORDER_ROUTING_KEY);
//        return new Queue(RabbitMqConstant.PAYMENT_ORDER_QUEUE,true,false,false,params);
//    }
//    @Bean
//    public Binding paymentOrderBinding() {
//        // 如果要让延迟队列之间有关联,这里的 routingKey 和 绑定的交换机很关键
//        return BindingBuilder.bind(paymentOrderQueue())
//                .to(paymentExchange())
//                .with(RabbitMqConstant.PAYMENT_ORDER_ROUTING_KEY);
//    }

    // 订阅过期死信队列
    @Bean
    public Queue paymentSubscriptionExpireDelayQueue(){
        return new Queue(RabbitMqConstant.PAYMENT_SUBSCRIPTION_EXPIRED_DELAY_QUEUE, true, false, false);
    }
    @Bean
    public Binding paymentSubscriptionExpireDelayBinding() {
        return BindingBuilder.bind(paymentSubscriptionExpireDelayQueue()).to(paymentDelayExchange()).with(RabbitMqConstant.PAYMENT_SUBSCRIPTION_EXPIRED_ROUTING_KEY);
    }
//    @Bean
//    public Queue paymentSubscriptionExpireQueue() {
//        Map<String, Object> params = new HashMap<>();
//        // x-dead-letter-exchange 声明了队列里的死信转发到的DLX名称，
//        params.put("x-dead-letter-exchange", RabbitMqConstant.DELAY_EXCHANGE);
//        // x-dead-letter-routing-key 声明了这些死信在转发时携带的 routing-key 名称。
//        params.put("x-dead-letter-routing-key", RabbitMqConstant.PAYMENT_SUBSCRIPTION_EXPIRED_DELAY_QUEUE);
//        return new Queue(RabbitMqConstant.PAYMENT_SUBSCRIPTION_EXPIRED_QUEUE,true,false,false,params);
//    }
//    @Bean
//    public Binding paymentSubscriptionExpireBinding() {
//        // 如果要让延迟队列之间有关联,这里的 routingKey 和 绑定的交换机很关键
//        return BindingBuilder.bind(paymentSubscriptionExpireQueue())
//                .to(paymentExchange())
//                .with(RabbitMqConstant.PAYMENT_SUBSCRIPTION_EXPIRED_ROUTING_KEY);
//    }

    // 普通队列配置
    /* 发送email队列 */
    @Bean
    public Queue sendEmailQueue() {
        return new Queue(RabbitMqConstant.EMAIL_SEND_QUEUE, true, false, false);
    }

    // 死信队列配置

    // 路由绑定配置
    /* 发送email路由绑定 */
    @Bean
    public Binding officeConvertBinding() {
        return new Binding(RabbitMqConstant.EMAIL_SEND_QUEUE,
                Binding.DestinationType.QUEUE,
                RabbitMqConstant.EMAIL_EXCHANGE,
                RabbitMqConstant.EMAIL_SEND_ROUTING_KEY, null);
    }

}
