package cn.kdan.cloud.pdf.office.email.rabbit.listener;

import cn.kdan.cloud.pdf.office.api.email.bo.EmailSendBO;
import cn.kdan.cloud.pdf.office.api.email.bo.TemplateHtmlBO;
import cn.kdan.cloud.pdf.office.common.constant.RabbitMqConstant;
import cn.kdan.cloud.pdf.office.common.exception.BackendRuntimeException;
import cn.kdan.cloud.pdf.office.common.utils.JsonUtils;
import cn.kdan.cloud.pdf.office.email.config.MailSenderConfig;
import cn.kdan.cloud.pdf.office.email.entity.EmailLog;
import cn.kdan.cloud.pdf.office.email.service.EmailLogService;
import cn.kdan.cloud.pdf.office.email.service.EmailTemplateService;
import cn.kdan.cloud.pdf.office.email.service.SendEmailHandlerService;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.rabbitmq.client.Channel;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.FileUtils;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.mail.javamail.JavaMailSenderImpl;
import org.springframework.mail.javamail.MimeMessageHelper;
import org.springframework.stereotype.Component;

import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;
import java.io.File;
import java.net.URL;
import java.util.Date;
import java.util.Map;
import java.util.Objects;

/**
 * @author ComPDFKit-WPH 2023/1/11
 */
        @Component
        @Slf4j
        @RequiredArgsConstructor
        public class SendEmailListener {

            private final SendEmailHandlerService sendEmailService;

            private final MailSenderConfig mailSenderConfig;
            private final EmailLogService emailLogService;
            private final EmailTemplateService emailTemplateService;

            /**
             * 发送邮件 监听
             *
             * @param message    message
             * @param channel    channel
             * @param emailSendBO sendObject
             */
            @RabbitListener(queues = RabbitMqConstant.EMAIL_SEND_QUEUE)
            public void backgroundApiAddQueue(Message message, Channel channel, EmailSendBO emailSendBO) {
                log.info("邮件发送监听内容：{}", emailSendBO);
                EmailLog emailLog = new EmailLog();
                String emailTemplateId = null;
                File file = null;
        try {
            if(StringUtils.isEmpty(emailSendBO.getToEmail())){
                log.info("邮箱为空：{}", emailSendBO);
                emailLog.setIsEnable(0);
                channel.basicAck(message.getMessageProperties().getDeliveryTag(), false);
                return;
            }
            TemplateHtmlBO template = emailTemplateService.getTemplateByUseEvent(emailSendBO.getUseEvent());
            emailTemplateId = template.getEmailTemplateId();
            JavaMailSenderImpl mailSender = mailSenderConfig.getDefaultSender();
            Map<String, String> sendContent = emailSendBO.getSendContent();
            String templateHtml = template.getTemplateHtml();
            for (String key : sendContent.keySet()) {
                templateHtml = templateHtml.replaceAll(key,sendContent.get(key));
            }
            String templateTitle = template.getTemplateTitle();
            Map<String, String> sendTitleContent = emailSendBO.getSendTitleContent();
            for (String key : sendTitleContent.keySet()) {
                templateTitle = templateTitle.replaceAll(key,sendTitleContent.get(key));
            }

            MimeMessage mimeMessage = mailSender.createMimeMessage();
            MimeMessageHelper helper = new MimeMessageHelper(mimeMessage, true);
            helper.setSubject(templateTitle);
            helper.setText(templateHtml, true);
            helper.setTo(emailSendBO.getToEmail());
            helper.setFrom(ObjectUtils.isEmpty(emailSendBO.getFromEmail())
                    ? Objects.requireNonNull(mailSender.getUsername())
                    : emailSendBO.getFromEmail(), "PDF Reader Pro");
            if(ObjectUtils.isNotEmpty(emailSendBO.getFileUrl())){
                file = new File(emailSendBO.getFileName());
                FileUtils.copyURLToFile(new URL(emailSendBO.getFileUrl()), file);
                helper.addAttachment(Objects.requireNonNull(emailSendBO.getFileName()), file);
            }
            mailSender.send(mimeMessage);
            emailLog.setIsEnable(1);
            channel.basicAck(message.getMessageProperties().getDeliveryTag(), false);
        } catch (Exception e) {
            log.error("消息消费失败:" + e.getMessage(), e);
            emailLog.setIsEnable(0);
            throw new BackendRuntimeException("消息消费失败:" + e.getMessage());
        } finally {
            emailLog.setSendTo(emailSendBO.getToEmail());
            emailLog.setSendFrom(emailSendBO.getFromEmail());
            emailLog.setSendTime(new Date());
            emailLog.setSendTemplate(emailTemplateId);
            emailLog.setSendContent(JsonUtils.getJsonString(emailSendBO.getSendContent()) + ";" + JsonUtils.getJsonString(emailSendBO.getSendTitleContent()));
            emailLogService.getBaseMapper().insert(emailLog);

            if (ObjectUtils.isNotEmpty(file) && file.exists()){
                file.delete();
            }
        }
    }


}
