package cn.kdan.cloud.pdf.office.payment.webhook;

import cn.kdan.cloud.pdf.office.api.payment.appstore.AppStoreReceiptInfo;
import cn.kdan.cloud.pdf.office.api.payment.constant.AppStoreAPIConstant;
import cn.kdan.cloud.pdf.office.api.payment.constant.PaddleAPIConstant;
import cn.kdan.cloud.pdf.office.api.payment.dto.AppStoreOrderSucceedDTO;
import cn.kdan.cloud.pdf.office.api.payment.dto.EquityVerificationDTO;
import cn.kdan.cloud.pdf.office.common.enums.ExceptionEnum;
import cn.kdan.cloud.pdf.office.common.exception.BackendRuntimeException;
import cn.kdan.cloud.pdf.office.common.utils.JsonUtils;
import cn.kdan.cloud.pdf.office.payment.error.ErrorMessage;
import cn.kdan.cloud.pdf.office.payment.properties.AppStoreProperties;
import cn.kdan.cloud.pdf.office.payment.service.AppStoreService;
import cn.kdan.cloud.pdf.office.payment.service.AppStoreWebhookService;
import cn.kdan.cloud.pdf.office.payment.service.PaddleWebhookService;
import cn.kdan.cloud.pdf.office.payment.utils.JwsUtil;
import cn.kdan.cloud.pdf.office.payment.webhook.appstore.NotificationDecoder;
import cn.kdan.cloud.pdf.office.payment.webhook.appstore.notification.*;
import com.fasterxml.jackson.core.JsonProcessingException;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.redisson.api.RLock;
import org.redisson.api.RedissonClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.*;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.client.RestTemplate;

import java.security.cert.CertificateException;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * @author ComPDFKit-WPH 2023/6/20
 */
@Controller
@RequestMapping("/appstore")
@RequiredArgsConstructor
@Slf4j
public class AppStoreWebhookMonitor {

    private final NotificationDecoder notificationDecoder = new NotificationDecoderImpl();

    private final RedissonClient redissonClient;

    private final AppStoreWebhookService appStoreWebhookService;
    private final RestTemplate restTemplate = new RestTemplate();

    private final AppStoreProperties properties;
    private final AppStoreService appStoreService;

    @RequestMapping(value = "/webhook/subscription", method = RequestMethod.POST)
    public ResponseEntity<String> receiveSubscriptionCallback(@RequestBody(required = false) ResponseBodyV2 responseBodyV2) {
        log.info("appstore 接收回调信息JWT值：{}", responseBodyV2);
        try {
            ResponseBodyV2DecodedPayload decodedPayload = notificationDecoder.decodePayload(responseBodyV2.getSignedPayload());
            log.info("appstore 接收回调信息decodedPayload值：{}", decodedPayload);
            String signedRenewalInfo = decodedPayload.getData().getSignedRenewalInfo();
            String signedTransactionInfo = decodedPayload.getData().getSignedTransactionInfo();

            JWSRenewalInfoDecodedPayload jwsRenewalInfoDecodedPayload = notificationDecoder.decodeRenewalInfo(signedRenewalInfo);
            JWSTransactionDecodedPayload jwsTransactionDecodedPayload = notificationDecoder.decodeTransaction(signedTransactionInfo);

            log.info("appstore 接收回调信息jwsRenewalInfoDecodedPayload值：{}", jwsRenewalInfoDecodedPayload);
            log.info("appstore 接收回调信息jwsTransactionDecodedPayload值：{}", jwsTransactionDecodedPayload);
            //
            if(jwsRenewalInfoDecodedPayload.getEnvironment().equals("Sandbox")){
                //如果当前服务不是沙盒环境（正式环境）就把请求转到测试环境
                if(!properties.getIsSandbox().equals("true")){
                    sandboxCallBack(responseBodyV2);
                    return ResponseEntity.status(HttpStatus.OK).build(); // 返回 200 OK
                }
            }
            String lockKey = decodedPayload.getNotificationType().toString().concat(decodedPayload.getNotificationUUID()) + "-RedissonLock";
            // 获取 RLock 对象
            RLock rLock = redissonClient.getLock(lockKey);
            try {
                // 上锁
                boolean resultLock = rLock.tryLock(30, 10, TimeUnit.SECONDS);
                // 业务逻辑开始
                if (resultLock) {
                    log.info("上锁成功, 当前事件：{}",decodedPayload.getNotificationType());
                    switch (decodedPayload.getNotificationType()) {
                        // 退款
                        case REFUND:
//                                appStoreWebhookService.subscriptionCancel(jwsRenewalInfoDecodedPayload.getOriginalTransactionId());
                            break;
                        // 订阅已成功续订
                        case DID_RENEW:
                            if (decodedPayload.getData().getStatus().equals(1)){
                                appStoreWebhookService.subscriptionPaymentSucceeded(false,jwsRenewalInfoDecodedPayload.getOriginalTransactionId(),jwsTransactionDecodedPayload.getTransactionId(),1,jwsTransactionDecodedPayload);
                            }
                            break;
                        // 订阅已过期
                        case DID_CHANGE_RENEWAL_STATUS:
                            if (Subtype.AUTO_RENEW_DISABLED.equals(decodedPayload.getSubtype())){
                                appStoreWebhookService.subscriptionCancel(jwsRenewalInfoDecodedPayload.getOriginalTransactionId());
                            }
                            break;
                        default:
                            log.info(ErrorMessage.PAYMENT_APPSTORE_WEBHOOK_EVENT_NO_HANDLE.concat(decodedPayload.getNotificationType().toString()));
//                            throw new BackendRuntimeException(ErrorMessage.PAYMENT_APPSTORE_WEBHOOK_EVENT_NO_HANDLE.concat(decodedPayload.getNotificationType().toString()));
                    }
                }
                return ResponseEntity.ok().build();
            } catch (InterruptedException e) {
                log.info("上锁失败");
                log.error("上锁失败".concat(e.getMessage()),e);
            } finally {
                if (rLock.isLocked() && rLock.isHeldByCurrentThread()) {
                    rLock.unlock();
                }
            }

        } catch (JsonProcessingException | CertificateException e) {
            log.error("appstore 回调失败：" + e.getMessage(), e);
            e.printStackTrace();
        }

        // 返回200 OK响应
        return ResponseEntity.ok().build();
    }

    public void sandboxCallBack(ResponseBodyV2 responseBodyV2) {
        // 将 responseBodyV2 转换为 JSON 字符串
        String jsonBody = JsonUtils.getJsonString(responseBodyV2); // 假设 JsonUtils.getJsonString 可以完成序列化
        // 创建 HttpEntity，包含 JSON 字符串和请求头
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON);
        HttpEntity<String> entity = new HttpEntity<>(jsonBody, headers); // getJsonHttpHeaders() 负责设置请求头
        // 使用 restTemplate 发送 POST 请求并检查响应状态码
        ResponseEntity<Void> response = restTemplate.exchange(
                properties.getTestCallBack(),  // URL
                HttpMethod.POST,            // 请求方法
                entity,                     // 请求体和头信息
                Void.class                  // 响应体类型，这里不需要实际的响应内容
        );
        if (response.getStatusCode()!= HttpStatus.OK) {
            log.error("appstore 回调测试环境失败：响应状态码不为 200");
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_APP_STORE_TRANSACTION_ID_VALIDATION_FAILED);
        }
    }
    public static void main(String[] args) {
//        String url = "https://api.storekit-sandbox.itunes.apple.com/inApps/v1/notifications/test";
        String url = "https://api.storekit-sandbox.itunes.apple.com/inApps/v1/notifications/test";
        // 设置请求头部
        HttpHeaders headers = new HttpHeaders();
        headers.set("Content-Type", "application/json");
        headers.set("Authorization", "Bearer YOUR_ACCESS_TOKEN");

        // 设置请求体
        String requestBody = "{\"notification_type\": \"TEST\"}";

        // 发送 POST 请求
        RestTemplate restTemplate = new RestTemplate();
        HttpEntity<String> request = new HttpEntity<>(requestBody, headers);
        ResponseEntity<String> response = restTemplate.exchange(url, HttpMethod.POST, request, String.class);

        // 处理响应
        HttpStatus statusCode = response.getStatusCode();
        String responseBody = response.getBody();
        System.out.println("Status Code: " + statusCode);
        System.out.println("Response Body: " + responseBody);
    }
}
