package cn.kdan.cloud.pdf.office.website.service.impl;

import cn.kdan.cloud.pdf.office.api.account.feign.UserApi;
import cn.kdan.cloud.pdf.office.api.account.feign.UserSubscriptionInfoApi;
import cn.kdan.cloud.pdf.office.api.account.vo.UserSubscriptionInfo;
import cn.kdan.cloud.pdf.office.api.payment.dto.*;
import cn.kdan.cloud.pdf.office.api.payment.feign.OrderApi;
import cn.kdan.cloud.pdf.office.api.payment.feign.SubscriptionApi;
import cn.kdan.cloud.pdf.office.api.payment.vo.CreateOrderResultVO;
import cn.kdan.cloud.pdf.office.api.payment.vo.SubscriptionSchemeVO;
import cn.kdan.cloud.pdf.office.api.payment.vo.SubscriptionsVO;
import cn.kdan.cloud.pdf.office.common.enums.ExceptionEnum;
import cn.kdan.cloud.pdf.office.common.enums.account.*;
import cn.kdan.cloud.pdf.office.common.exception.BackendRuntimeException;
import cn.kdan.cloud.pdf.office.common.pojo.ResultMap;
import cn.kdan.cloud.pdf.office.common.utils.MyDateUtils;
import cn.kdan.cloud.pdf.office.common.vo.UserInfoVO;
import cn.kdan.cloud.pdf.office.common.vo.UserVO;
import cn.kdan.cloud.pdf.office.website.dto.IsSubscriptionDTO;
import cn.kdan.cloud.pdf.office.website.service.PaymentService;
import cn.kdan.cloud.pdf.office.website.utils.SecurityUtils;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Map;

/**
 * @author ComPDFKit-WPH 2023/2/9
 */
@Service
@Slf4j
@RequiredArgsConstructor
@Transactional
public class PaymentServiceImpl implements PaymentService {

    private final SubscriptionApi subscriptionApi;

    private final OrderApi orderApi;

    private final UserApi userApi;

    private final UserSubscriptionInfoApi userSubscriptionInfoApi;

    @Override
    public ResultMap<Map<String,SubscriptionSchemeVO>> getSubscriptionSchemeInfo(String appId) {
        String userId = SecurityUtils.getUserId();
        return subscriptionApi.getSubscriptionSchemeInfo(userId,appId);
    }

    @Override
    public LocalDateTime getSubscriptionEndDate(Integer platform) {
        String userId = SecurityUtils.getUserId();
        return MyDateUtils.dateToLocalDateTime(subscriptionApi.getSubscriptions(userId, platform).getResult().getEndDate());
    }

    @Override
    public CreateOrderResultVO createOrder(CreateOrderDTO createOrderDTO) {
        UserVO userVO = userApi.getByAppAccount(createOrderDTO.getEmail(), PlatformTypeEnum.WEBSITE.value().toString()).getResult();
        if(!ObjectUtils.isEmpty(userVO)){
            //如果用户已经有该产品的订阅
            UserSubscriptionInfo userSubscriptionInfo = userSubscriptionInfoApi.getByUserIdAndProductId(userVO.getId(),createOrderDTO.getProductId()).getResult();
            if(!ObjectUtils.isEmpty(userSubscriptionInfo)&& userSubscriptionInfo.getStatus().equals(PDFOfficeUserSubscriptionStatusEnum.SUBSCRIPTION_IN_PROGRESS.value())
                    && userSubscriptionInfo.getPayType().equals(PayTypeEnum.AUTO.value())){
                throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_REPEAT_SUBSCRIPTION);
            }
            // 查询订阅信息，有返回false，无返回true
            List<SubscriptionsVO> subscriptionsVOS = subscriptionApi.getSubscriptionsByUserIdAndProductId(userVO.getId(),createOrderDTO.getProductId()).getResult();
            if (CollectionUtils.isEmpty(subscriptionsVOS) || subscriptionsVOS.size() == 0){
                //1折扣0不折扣
                createOrderDTO.setIsDiscount(1);
            }else{
                createOrderDTO.setIsDiscount(0);
            }
            createOrderDTO.setUserId(userVO.getId());
        }else {
            createOrderDTO.setIsDiscount(1);
        }
        CreateOrderResultVO vo = orderApi.createOrder(createOrderDTO).getResult();
        vo.setResult(null);
        return vo;
    }

    @Override
    public void cancelSubscription(String productId) {
        String userId = SecurityUtils.getUserId();
        //获取用户对于某个产品订阅列表
        SubscriptionsVO subscriptionsVO = subscriptionApi.getLatestSubscriptionForActualPayment(userId,productId).getResult();
        //用户对于某个产品活跃的订阅只有一个，根据订阅id去取消续订
        CancelSubscriptionDTO dto = new CancelSubscriptionDTO();
        if(!ObjectUtils.isEmpty(subscriptionsVO)){
            dto.setSubscriptionId(subscriptionsVO.getId());
            orderApi.cancelSubscription(dto);
        }
    }

    @Override
    public CreateOrderResultVO appStoreCreateOrder(CreateOrderDTO createOrderDTO) {
        return orderApi.appStoreCreateOrder(createOrderDTO).getResult();
    }


    /**
     * appStore 权益校验
     *
     * @param equityVerificationDTO equityVerificationDTO
     * @return UserInfoVO
     */
    @Override
    public UserInfoVO appStoreEquityVerification(EquityVerificationDTO equityVerificationDTO){
        return orderApi.appStoreEquityVerification(equityVerificationDTO).getResult();
    }

    @Override
    public UserInfoVO googleEquityVerification(GooglePayDTO googlePayDTO){
        return orderApi.googleEquityVerification(googlePayDTO).getResult();
    }

    @Override
    public boolean isSubscription(IsSubscriptionDTO isSubscriptionDTO) {
        // 判断用户是否存在
        UserVO userVO = userApi.getByEmail(isSubscriptionDTO.getEmail()).getResult();
        if (ObjectUtils.isEmpty(userVO)) {
            return true;
        }
        if (!PDFOfficeUserStatusEnum.VALID.value().toString().equals(userVO.getValidFlag())&&!PDFOfficeUserStatusEnum.LOGGED_OUT.value().toString().equals(userVO.getValidFlag())){
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_ABNORMAL_USER_STATUS);
        }
        // 查询订阅信息，有返回false，无返回true
        List<SubscriptionsVO> subscriptionsVOS = subscriptionApi.getSubscriptionsByUserIdAndProductId(userVO.getId(),isSubscriptionDTO.getProductId()).getResult();
        if (CollectionUtils.isEmpty(subscriptionsVOS) || subscriptionsVOS.size()<=0){
            return true;
        }
        return false;
    }


}
